@@ -2,6 +2,9 @@
 
 # solves two by two Ax=c linear system and returns the optimal values defined by
 # constraints. This is thanks to the convexity of (weighted) least squared error
+from collections import deque
+
+
 def solve2_2(node, a_11, a_12, a_21, a_22, c_1, c_2, negative_branch):
     edge_length = node.edge_length
     det = 1 / (a_11 * a_22 - a_12 * a_21)
@@ -34,10 +37,22 @@ def solve2_2(node, a_11, a_12, a_21, a_22, c_1, c_2, negative_branch):
     node.x_1_neg = x_1_neg
     node.x_2_neg = x_2_neg
 
+
 def index_edges(tree):
     counter=0
     for node in tree.traverse_postorder():
         node.edge_index = counter
         counter += 1
-        node.valid = True
-    tree.root.valid = False
+        node.valid = False
+
+
+def set_levels(tree):
+    root = tree.root
+    root.level = 0
+    q = deque()
+    q.append(root)
+    while len(q) != 0:
+        n = q.popleft()
+        for c in n.children:
+            c.level = n.level + 1
+        q.extend(n.children)
